/*
Release v0.3 dd 17 may 2021 Software for calibrating MPU6050
Connect the MPU6050 board to an arduino with USB interface (output is via USB interface)
--> Note: this can be e.g. HW_sensorboard_MPU6050 (if this board has an USB interface e.g. an Arduino Nano)
Connect Vcc and GND and SCL to A5, and SDA to A4.
Place the MPU6050 board as horizonal as possible with componentside up. Don't touch the board during calibration!
Output is via USB interface 9600 BPS

Some parts of this programm are based on publication of Kristian Lauszus, TKJ Electronics  

This is open software: GNU3
 
Comments are welcome at: ON4CDU(at)uba.be
*/

#include <Wire.h>

/* IMU Data */
float accX, accY, accZ;

long tempx,tempy,tempz;
byte i2cData[14]; // Buffer for I2C data

int nax_offset = 0;
int nay_offset = 0;
int naz_offset = 0;

byte IMUAddress = 0x68; // AD0 is logic low on the PCB
unsigned int I2C_TIMEOUT = 1000; // Used to check for errors in I2C communication


int i2cWrite(byte registerAddress, byte data, boolean sendStop) {
  return i2cWrite(registerAddress, &data, 1, sendStop); // Returns 0 on success
}

int i2cWrite(byte registerAddress, byte *data, byte length, boolean sendStop) {
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  Wire.write(data, length);
  byte rcode = Wire.endTransmission(sendStop); // Returns 0 on success
  if (rcode) {
    Serial.print(F("i2cWrite failed: "));
    Serial.println(rcode);
  }
  return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
}

int i2cRead(byte registerAddress, byte *data, byte nbytes) {
  unsigned long timeOutTimer;
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  byte rcode = Wire.endTransmission(false); // Don't release the bus
  if (rcode) {
    Serial.print(F("i2cRead failed: "));
    Serial.println(rcode);
    return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
  }

  Wire.requestFrom(IMUAddress, nbytes, (byte)true); // Send a repeated start and then release the bus after reading
  for (byte i = 0; i < nbytes; i++) {
    if (Wire.available())
      data[i] = Wire.read();
    else {
      timeOutTimer = micros();
      while (((micros() - timeOutTimer) < I2C_TIMEOUT) && !Wire.available());
      if (Wire.available())
        data[i] = Wire.read();
      else {
        Serial.println(F("i2cRead timeout"));
        return 5; // This error value is not already taken by endTransmission
      }
    }
  }
  return 0; // Success
}

void setup() {
  Serial.begin(9600);
  Wire.begin();
  i2cData[0] = 7; // Set the sample rate to 1 KHz
  i2cData[1] = 0x06; // Disable FSYNC and set 5 Hz Acc filtering, 5 Hz Gyro filtering, 1 KHz sampling
  i2cData[2] = 0x00; // Set Gyro Full Scale Range to ±250deg/s
  i2cData[3] = 0x00; // Set Accelerometer Full Scale Range to ±2g
  while (i2cWrite(0x19, i2cData, 4, false)); // Write to all four registers at once
  while (i2cWrite(0x6B, 0x01, true)); // PLL with X axis gyroscope reference and disable sleep mode

  while (i2cRead(0x75, i2cData, 1));
  if (i2cData[0] != 0x68) { // Read "WHO_AM_I" register
    Serial.print(F("Error reading sensor"));
    while (1);
  }
  delay(1000);
  Serial.println (" starting calibration procedure          ");
  Serial.println ("Place the MPU6050 board as horizonal as possible with componentside up. Don't touch the board during calibration!");
  Serial.println ("Normaly the calibration procedure will finish within 2 minutes automaticly, if not look to the accX and accY values");
  delay(1000);
  

  while (i2cRead(0x06, i2cData, 6));
  int xoff = ((i2cData[0] << 8) | i2cData[1]);
  int yoff = ((i2cData[2] << 8) | i2cData[3]);
  int zoff = ((i2cData[4] << 8) | i2cData[5]);
 
  delay(50);
  Serial.print (" your old offset are: x-offset  y-offset  z-offset  ");
  Serial.print (xoff); Serial.print("\t");
  Serial.print (yoff); Serial.print("\t");
  Serial.println (zoff);

  while (Serial.available() && Serial.read()); // empty buffer
  while (!Serial.available()){
    Serial.println(F("Send a character to start a new calibration cycle\n"));
    delay(2500);
  }                
  while (Serial.available() && Serial.read()); // empty buffer again

// reset offsets
  for (int i = 0; i<6; i++) {
    i2cData[i] = 0;
  }
  while (i2cWrite(0x06, i2cData, 6, false));
  Serial.println("reseting offsets"); 

  delay(5000); // Wait for sensor to stabilize 
}

void loop() {
  
/* Update all the values */
for (int j=0; j<80; j++){
 
  while (i2cRead(0x3B, i2cData, 12));
  accX = ((i2cData[0] << 8) | i2cData[1]);
  accY = ((i2cData[2] << 8) | i2cData[3]);
  accZ = ((i2cData[4] << 8) | i2cData[5]);

// simple low pass filter
  tempx = tempx - (tempx>>7) + accX;
  accX = (tempx>>7);
  tempy = tempy - (tempy>>7) + accY;
  accY = (tempy>>7);
  tempz = tempz - (tempz>>7) + accZ;
  accZ = (tempz>>7); 
}

  Serial.println("Calibrated when ACC values of accX and accY are close to zero (+- 10); accZ should read about 16384");
  Serial.print("  accX: "); Serial.print (int(accX));   
  Serial.print("  accY: "); Serial.print (int(accY));
  Serial.print("  accZ: "); Serial.print (int(accZ));

  int ax_offset=-accX/8;  
  nax_offset = nax_offset + ax_offset;
  int ay_offset=-accY/8;
  nay_offset = nay_offset + ay_offset;
  int az_offset=(16384 - accZ)/8;
  naz_offset = naz_offset + az_offset;

  i2cData[0] = nax_offset>>8;
  i2cData[1] = nax_offset&0xFF;
  i2cData[2] = nay_offset>>8;
  i2cData[3] = nay_offset&0xFF;
  i2cData[4] = naz_offset>>8;
  i2cData[5] = naz_offset&0xFF;
  
  while (i2cWrite(0x06, i2cData, 6, false));
  while (i2cRead(0x06, i2cData, 6));
  int xoff = ((i2cData[0] << 8) | i2cData[1]);
  int yoff = ((i2cData[2] << 8) | i2cData[3]);
  int zoff = ((i2cData[4] << 8) | i2cData[5]);
 
  Serial.print("   your new offset are: x-offset: "); Serial.print (xoff);   
  Serial.print("  y-offset:  "); Serial.print (yoff);
  Serial.print("  z-offset:  "); Serial.println (zoff);
  if ((abs(accX) < 5 ) && (abs(accY) < 5 ) && (abs(accZ - 16384) < 7)){
    while (1){
      Serial.println ("calibration succesfull, calibration procedure finished");
      delay (2000);
    }
   }
 
  delay(1000);
}


  
