/* SW_display_analog_v0.3 dd 24 august 2016
 Program for displaying AZ (azimuth) and EL (elevation) values on LCD (16 x 2) screen using Arduino
 This program can be used for a display unit with one or two RS485 inputs 
 
 This program can be used in combination with the (analog) sensorboard:
 -- HW_sensorboard_analog: with 1 or 2 potentiometers as input (azimuth and elevation data)
  -----------------------------------------------------------------------------------------------------------
  | Note: for correct display of values for sensorboard HW_sensorboard_HH12 or HW_sensorboard_MPU6050
  | it is recommended to use SW_display_digital                                                                                   |
  -----------------------------------------------------------------------------------------------------------  
 Input data to the display unit is coming from a sensorpboard via RS485: RS485 96000 N 8 1 
 The RS485 9600 bits/s frames have the following byte layout: del elhi ello azhi azlo frame_end
 del(imiter) is:
	-- 0xD0 (hex) for frames with only az information
	-- 0xD1 for for frames with only el information
	-- 0xD2 for for frames with az and el information
 The azimuth and elevation data is split into 2 (highbyte, lowbyte) bytes
 The frame is ending with 0x13 (hex)
 The frame layout is del elhi ello azhi azlo 0x13 
  
 The display resolution for azimuth and elevation can separatly be set to: 
 1 or .5 or .2 or .1 degree

 For correct displaying azimuth and elevation values a convertion factor
 for the encoder rotation against the antenna rotation
 has to be filled in: azgearfactor and elgearfactor
 The offset for azimuth and elevation can be set with the external push buttons

 For azimuth and elevation also the rotation direction can be changed:
 --> look into the 'transformaz' and 'transformel' routine to make a reverse scaling 

 input is via a max485 coupled to the Arduino usart
 connections of LCD display and offsetbuttons: see program
 
 This is open software: GNU3
 
 comments are welcome. ON4CDU(at)uba.be
*/

#include <LiquidCrystal.h> // include the library code:
#include <EEPROM.h>
#include <stdlib.h>

const int offazup = 3;  // pin for change az offset
const int offazdn = 4;  // pin for change az offset
const int offelup = 5;  // pin for change el offset
const int offeldn = 6;  // pin for change el offset

// button variables
int stateoffazup = 1;  // buttonstate az offset
int stateoffazdn = 1;  // buttonstate
int stateoffelup = 1;  // buttonstate el offset
int stateoffeldn = 1;  // buttonstate

// frame variables
unsigned char del = 0; // delimiter character
unsigned char frameend = 0; // end of frame character
unsigned char ello = 0; // elevation low byte
unsigned char elhi = 0; // elevation high byte
unsigned char azlo = 0; // azimuth low byte
unsigned char azhi = 0; // azimuth high byte

//display resolution az; MAKE YOUR OWN CHOISE
//int azr = 1; // display resolution 1 degree
//int azr = 2; // display resolution .5 degree
//int azr = 5; // display resolution .2 degree
int azr = 10; // display resolution .1 degree

//display resolution el; MAKE YOUR OWN CHOISE
//int elr = 1; // display resolution 1 degree
//int elr = 2; // display resolution .5 degree
//int elr = 5; // display resolution .2 degree
int elr = 10; // display resolution .1 degree

// calculation and display variables
int el = 0; // elevation value in decimal format
float elg = 0; // el in degrees
int elgi = 0; // el in integer format (10 times el)
float elgearfactor = 1.0; // FILL IN YOUR CORRECTION FACTOR FOR THE ELEVATION GEAR 

int az = 0; // azimuth value in decimal format
float azg = 0; // az in degrees
int azgi= 0; // az in integer format
float azgearfactor = 1.0; // e.g. 1.764; // FILL IN YOUR CORRECTION FACTOR FOR THE AZIMUTH GEAR 

int offsetaz = 0; // offset for azimuth
int offsetel = 0; // offset for elevation

// EEPROM addresses
#define EEPROM_ADDR_STATUS 0
#define EEPROM_ADDR_OFFSET_AZ 1
#define EEPROM_ADDR_OFFSET_EL 3

// general variables needed in program
int d = 0; // timer delay

LiquidCrystal lcd(12, 11, 10, 9, 8, 7);// initialize the library with the numbers of the interface pins

void setup()
{
  lcd.begin(16, 2); // set up the LCD's number of columns and rows
  Serial.begin(9600);  // opens serial port, sets data rate to 9600 bps
  pinMode(offazup, INPUT_PULLUP); // set pin to input and enable pullup resistor
  pinMode(offazdn, INPUT_PULLUP);
  pinMode(offelup, INPUT_PULLUP);
  pinMode(offeldn, INPUT_PULLUP);

  if (1 == EEPROM.read (EEPROM_ADDR_STATUS))
  {
    char* p_mem = (char*)(&offsetaz);
    p_mem [0] = EEPROM.read (EEPROM_ADDR_OFFSET_AZ);
    p_mem [1] = EEPROM.read (EEPROM_ADDR_OFFSET_AZ + 1);
    p_mem = (char*)(&offsetel);
    p_mem [0] = EEPROM.read (EEPROM_ADDR_OFFSET_EL);
    p_mem [1] = EEPROM.read (EEPROM_ADDR_OFFSET_EL + 1);
  }
}

// Main program
void loop()
{
  readstatebuttons();// changing offset?
  d = 300;
  while (stateoffazup == LOW || stateoffazdn == LOW || stateoffelup == LOW || stateoffeldn == LOW) {
    offsetcorrection();
  }

  if (Serial.available() > 15)
  {
    del = Serial.read();
    Serial.println (del);
    while (del != 0xD0 && del != 0xD1 && del != 0xD2 ) {
      del = Serial.read();
    }; // search for 0xD0, 0xD1 or 0xD2 hex in data

    if (del == 0xD0) // header az frame
    {
      del = 0;
      readframe();
      if (frameend == 0x13 && elhi == 0 && ello == 0) // check frame
      {
        transformaz();
        displayaz();
      }
    }

    if (del == 0xD1)  // header of el frame
    {
      del = 0;
      readframe();
      if (frameend == 0x13 && azhi == 0 && azlo == 0) //check frame
      {
        transformel();
        displayel();
      }
    }

    if (del == 0xD2)  // header of az/el frame
    {
      del = 0;
      readframe();
      if (frameend == 0x13) //end of frame character
      {
        transformaz();
        displayaz();
        transformel();
        displayel();
      }
    }
  }
}

void readframe()  // read frame
{
  elhi = Serial.read();
  ello = Serial.read();
  azhi = Serial.read();
  azlo = Serial.read();
  frameend = Serial.read();
}

void transformaz()  // transform azimuth info to degrees
{
  az = azhi * 256 + azlo; // az information in decimal format
//az = 4092 - az; // for reverse scaling
  azg =(float(az) * 360 * azgearfactor)/4093; // transformation to degrees
  azg = azr* azg;
  azg = round (azg);
  azg = azg/azr;
  azg = 10 * azg;
  azgi = azg;
}

void transformel()  // transform elevation info to degrees
{
  el = elhi * 256 + ello;
//el = 4092 - el; // for reverse scaling
  elg = (float(el) * 360 * elgearfactor)/4093;
  
  elg = elr * elg;
  elg = round (elg);
  elg = elg/elr;
  elg = 10 * elg;
  elgi = elg;
  
  }

void displayaz() // display azimuth value
{
  int azgd; // az value for display purposes
  char buffer [17]; //buffer for dislay a line on LCD
  
  azgd = azgi + (offsetaz)/azr;
 
//Serial.println (offsetaz); // for test purposes

  if (azgd >= 3600) {
    azgd = azgd - 3600;
  }
  else if (azgd < 0) {
    azgd = 3600 + azgd; // az has always a positive value
  }
 
  int upper_part = azgd/10;
  int lower_part = azgd%10; 

//Serial.println(upper_part); // for test purposes
//Serial.println(lower_part); // for test purposes
 
  if (azr <=1) {
    sprintf (buffer, "Azimuth   %3d ", upper_part);
  }
  else {
    sprintf (buffer, "Azimuth   %3d.%1d ", upper_part, lower_part);
  }
  
  lcd.setCursor (0, 0);
  lcd.print (buffer);
}

void displayel()  // display elevation value
{
  int elgd; // el value for display purposes
  int neg = 0; // boolean for display negative el values
  char buffer [17]; //buffer for dislay a line on LCD
   
  elgd = elgi + offsetel/elr; // display value corrected with offset
 
if (elgd >= 3600) {
    elgd = elgd - 3600;
  }
  else if (elgd < 0) {
    elgd = 3600 + elgd; // el has now always a positive value
  }
   
if (elgd > 2700) {
    elgd = 3600 - elgd;
    neg = 1;
  }

  int upper_part = elgd/10;
  int lower_part = elgd % 10;
  
//Serial.println(upper_part); // for test purposes
//Serial.println(lower_part); // for test purposes
   
 if (neg==1) {
  if (elr <=1){
    sprintf (buffer, "Elevation -%2d", upper_part);
  }
  else {
    sprintf (buffer, "Elevation -%2d.%1d ", upper_part, lower_part);
  }
    neg = 0;
 }
 else {
  if (elr <=1){
   sprintf (buffer, "Elevation %3d", upper_part); 
  }
  else {
    sprintf (buffer, "Elevation %3d.%1d ", upper_part, lower_part);
  }
  
 }
  lcd.setCursor (0, 1);
  lcd.print (buffer);
}

void readstatebuttons()
{
  stateoffazup = digitalRead(offazup);
  stateoffazdn = digitalRead(offazdn);
  stateoffelup = digitalRead(offelup);
  stateoffeldn = digitalRead(offeldn);
}

void offsetcorrection()
{
  if (stateoffazup == LOW)
  {
    offsetaz = offsetaz + 10;
    displayaz();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffazdn == LOW)
  {
    offsetaz = offsetaz - 10;
    displayaz();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffelup == LOW)
  {
    offsetel = offsetel + 10;
    displayel();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffeldn == LOW)
  {
    offsetel = offsetel - 10;
    displayel();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  
//offsetaz = 0; //useful for filling offset zeros in EEPROM
//offsetel = 0; //useful for filling offset zeros in EEPROM

  // writing offset data to EEPROM
  EEPROM.write (EEPROM_ADDR_STATUS, 1);
  char* p_mem = (char*)(&offsetaz);
  EEPROM.write (EEPROM_ADDR_OFFSET_AZ  , p_mem [0]);
  EEPROM.write (EEPROM_ADDR_OFFSET_AZ + 1, p_mem [1]);
  p_mem = (char*)(&offsetel);
  EEPROM.write (EEPROM_ADDR_OFFSET_EL  , p_mem [0]);
  EEPROM.write (EEPROM_ADDR_OFFSET_EL + 1, p_mem [1]);

  readstatebuttons();
}
